package com.example.weathrmax;

import android.Manifest;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.provider.Settings;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.StringRequest;
import com.android.volley.toolbox.Volley;

import org.json.JSONArray;
import org.json.JSONObject;
import org.w3c.dom.Text;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

public class MainActivity extends AppCompatActivity implements LocationListener {

    Context context;

    Button autoButton;
    Button manualButton;
    Button randomButton;
    TextView manualText;

    public LocationManager locationManager;
    public Location location;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        context = this;
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        autoButton = (Button) findViewById(R.id.autoBtn);
        manualButton = (Button) findViewById(R.id.manualBtn);
        randomButton = (Button) findViewById(R.id.randomBtn);
        manualText = (TextView) findViewById(R.id.manualText);

        checkPermissions();
        // If available, initialize weather on user's location
        getUserLocation();

        autoButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                try {
                    getUserLocation();
                }
                catch(Exception ex){
                    ex.printStackTrace();
                }
            }
        });

        manualButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                try {
                    String cityName = manualText.getText().toString();
                    if(cityName != null && !cityName.trim().isEmpty()){
                        Intent i = new Intent(MainActivity.this, WeatherActivity.class);
                        i.putExtra("NAME", cityName);
                        i.putExtra("TYPE", "CITY");
                        MainActivity.this.startActivity(i);
                    }
                    else{
                        manualText.setError("Field cannot be empty");
                    }
                }
                catch(Exception ex){
                    ex.printStackTrace();
                }
            }
        });

        randomButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                try {
                    Intent i = new Intent(MainActivity.this, WeatherActivity.class);
                    i.putExtra("TYPE", "RANDOM");
                    MainActivity.this.startActivity(i);
                }
                catch(Exception ex){
                    ex.printStackTrace();
                }
            }
        });
    }

    public void getUserLocation(){
        // Sometimes returns null? If it does, try the other methods.
        location = getBestLocation();

        if (location == null){
            location = getLocationMiika();
        }
        if (location == null){
            location = getLocationRadu();
        }
        if (location != null) {
            Intent i = new Intent(MainActivity.this, WeatherActivity.class);
            i.putExtra("TYPE", "COORDS");
            i.putExtra("LAT", location.getLatitude());
            i.putExtra("LON", location.getLongitude());
            MainActivity.this.startActivity(i);
        } else {
            Toast.makeText(context, "Failed to get location!\nPlease check that device location is enabled.", Toast.LENGTH_SHORT).show();
            Log.d("LOCATION", "location = " + location);
        }
    }

    public void checkPermissions(){
        if(ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION)
                != PackageManager.PERMISSION_GRANTED){
            ActivityCompat.requestPermissions(this, new String[]{
                            Manifest.permission.ACCESS_COARSE_LOCATION
                    },
                    2);
        }

        if(ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED){
            ActivityCompat.requestPermissions(this, new String[]{
                            Manifest.permission.ACCESS_FINE_LOCATION
                    },
                    2);
        }

        if(ContextCompat.checkSelfPermission(this, Manifest.permission.INTERNET)
                != PackageManager.PERMISSION_GRANTED){
            ActivityCompat.requestPermissions(this, new String[]{
                            Manifest.permission.INTERNET
                    },
                    2);
        }
    }

    // https://stackoverflow.com/questions/20438627/getlastknownlocation-returns-null
    public Location getBestLocation() {
        Location bestLocation = null;
        try {
            locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
            List<String> providers = locationManager.getProviders(true);

            for (String provider : providers) {
                Location l = locationManager.getLastKnownLocation(provider);
                //Location l = locationManager.requestLocationUpdates(provider, );
                Log.d("PROVIDERS", "Provider: " + provider + ", location:" + l);
                if (l == null) {
                    continue;
                }
                if (bestLocation == null || l.getAccuracy() > bestLocation.getAccuracy()) {
                    bestLocation = l;
                }
            }
        }
        catch(SecurityException e){
            e.printStackTrace();
        }
        return bestLocation;
    }

    public Location getLocationMiika() {
        Location _location = null;
        try {
            locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
            _location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);

        }
        catch(SecurityException e){
            e.printStackTrace();
        }
        return _location;
    }

    public Location getLocationRadu() {
        try {
            locationManager = (LocationManager)getSystemService(LOCATION_SERVICE);

            boolean isGPSEnabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
            boolean isNetworkEnabled = locationManager
                    .isProviderEnabled(LocationManager.NETWORK_PROVIDER);

            if (isGPSEnabled || isNetworkEnabled) {
                if (isGPSEnabled) {
                    if (location == null) {
                        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 1000, 0, this);
                        if (locationManager != null) {
                            location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
                        }
                    }
                }
                if (location==null && isNetworkEnabled) {
                    locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 1000, 0, this);
                    if (locationManager != null) {
                        location = locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
                    }
                }
            }

        } catch (SecurityException e) {
            e.printStackTrace();
        }

        return location;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public void onLocationChanged(Location location) {

    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {

    }

    @Override
    public void onProviderEnabled(String s) {

    }

    @Override
    public void onProviderDisabled(String s) {

    }
}
