package com.example.weathrmax;

import android.content.Context;
import android.content.DialogInterface;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;

import org.json.JSONArray;
import org.json.JSONObject;
import org.w3c.dom.Text;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Random;

public class WeatherActivity extends AppCompatActivity {

    private Context context;

    private TextView cityText;
    private TextView coordsText;
    private TextView countryText;
    private TextView tempText;
    private TextView descText;
    private ImageView weatherIcon;
    private LinearLayout forecastLayout;
    private TextView forecastTitle;

    public LocationManager locationManager;
    private RequestQueue queue;
    private final String baseUrl = "http://api.openweathermap.org/data/2.5/";
    private final String units = "units=metric";
    private final String key = "APPID=f76ba7dfe9b0bbfb243db62351079947";
    private final Random random = new Random();

    //region Capital list, https://en.wikipedia.org/wiki/List_of_national_capitals_in_alphabetical_order
    final List<String> capitals = Arrays.asList("Abu Dhabi",
            "Abuja",
            "Accra",
            "Adamstown",
            "Addis Ababa",
            "Algiers",
            "Alofi",
            "Amman",
            "Amsterdam",
            "Andorra la Vella",
            "Ankara",
            "Antananarivo",
            "Apia",
            "Ashgabat",
            "Asmara",
            "Astana",
            "Asunción",
            "Athens",
            "Avarua",
            "Baghdad",
            "Baku",
            "Bamako",
            "Bandar Seri Begawan",
            "Bangkok",
            "Bangui",
            "Banjul",
            "Basseterre",
            "Beijing",
            "Beirut",
            "Belgrade",
            "Belmopan",
            "Berlin",
            "Bern",
            "Bishkek",
            "Bissau",
            "Bogotá",
            "Brasília",
            "Bratislava",
            "Brazzaville",
            "Bridgetown",
            "Brussels",
            "Bucharest",
            "Budapest",
            "Buenos Aires",
            "Bujumbura",
            "Cairo",
            "Canberra",
            "Caracas",
            "Castries",
            "Charlotte Amalie",
            "Chișinău",
            "Cockburn Town",
            "Conakry",
            "Copenhagen",
            "Dakar",
            "Damascus",
            "Dhaka",
            "Dili",
            "Djibouti",
            "Dodoma",
            "Doha",
            "Douglas",
            "Dublin",
            "Dushanbe",
            "Edinburgh of the Seven Seas",
            "El Aioun",
            "Tifariti",
            "Episkopi Cantonment",
            "Flying Fish Cove",
            "Freetown",
            "Funafuti",
            "Gaborone",
            "George Town",
            "Georgetown",
            "Gibraltar",
            "Guatemala City",
            "Gustavia",
            "Hagåtña",
            "Hamilton",
            "Hanga Roa",
            "Hanoi",
            "Harare",
            "Hargeisa",
            "Havana",
            "Helsinki",
            "Hong Kong",
            "Honiara",
            "Islamabad",
            "Jakarta",
            "Jamestown",
            "Jerusalem",
            "Juba",
            "Kabul",
            "Kampala",
            "Kathmandu",
            "Khartoum",
            "Kiev",
            "Kigali",
            "King Edward Point",
            "Kingston",
            "Kingston",
            "Kingstown",
            "Kinshasa",
            "Kuala Lumpur",
            "Putrajaya",
            "Kuwait City",
            "Libreville",
            "Lilongwe",
            "Lima",
            "Lisbon",
            "Ljubljana",
            "Lomé",
            "London",
            "Luanda",
            "Lusaka",
            "Luxembourg",
            "Madrid",
            "Majuro",
            "Malabo",
            "Malé",
            "Managua",
            "Manama",
            "Manila",
            "Maputo",
            "Marigot",
            "Maseru",
            "Mata-Utu",
            "Mbabane",
            "Lobamba",
            "Mexico City",
            "Minsk",
            "Mogadishu",
            "Monaco",
            "Monrovia",
            "Montevideo",
            "Moroni",
            "Moscow",
            "Muscat",
            "N'Djamena",
            "Nairobi",
            "Nassau",
            "Naypyidaw",
            "New Delhi",
            "Ngerulmud",
            "Niamey",
            "Nouakchott",
            "Nouméa",
            "Nukuʻalofa",
            "Nuuk",
            "Oranjestad",
            "Oslo",
            "Ottawa",
            "Ouagadougou",
            "Pago Pago",
            "Palikir",
            "Panama City",
            "Papeete",
            "Paramaribo",
            "Paris",
            "Philipsburg",
            "Phnom Penh",
            "Plymouth",
            "Brades Estate",
            "Podgorica",
            "Cetinje",
            "Port Louis",
            "Port Moresby",
            "Port of Spain",
            "Port Vila",
            "Port-au-Prince",
            "Porto-Novo",
            "Cotonou",
            "Prague",
            "Praia",
            "Pretoria",
            "Bloemfontein",
            "Cape Town",
            "Pristina",
            "Pyongyang",
            "Quito",
            "Rabat",
            "Reykjavík",
            "Riga",
            "Riyadh",
            "Road Town",
            "Rome",
            "Roseau",
            "Saipan",
            "San José",
            "San Juan",
            "San Marino",
            "San Salvador",
            "Sana'a",
            "Aden",
            "Santiago",
            "Santo Domingo",
            "Sarajevo",
            "Seoul",
            "Singapore",
            "Skopje",
            "Sofia",
            "Sri Jayawardenepura Kotte",
            "Colombo",
            "St. George's",
            "St. Helier",
            "St. John's",
            "St. Peter Port",
            "St. Pierre",
            "Stanley",
            "Stepanakert",
            "Stockholm",
            "Sucre",
            "La Paz",
            "Sukhumi",
            "Suva",
            "São Tomé",
            "Taipei",
            "Tallinn",
            "Tarawa",
            "Tashkent",
            "Tbilisi",
            "Kutaisi",
            "Tegucigalpa",
            "Tehran",
            "The Valley",
            "Thimphu",
            "Tirana",
            "Tiraspol",
            "Tokyo",
            "Tripoli",
            "Tskhinvali",
            "Tunis",
            "Tórshavn",
            "Ulaanbaatar",
            "Vaduz",
            "Valletta",
            "Vatican City",
            "Victoria",
            "Vienna",
            "Vientiane",
            "Vilnius",
            "Warsaw",
            "Washington D.C.",
            "Wellington",
            "West Island",
            "Willemstad",
            "Windhoek",
            "Yamoussoukro",
            "Abidjan",
            "Yaoundé",
            "Yaren",
            "Yerevan",
            "Zagreb"
            );
    //endregion

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        context = this;
        setContentView(R.layout.activity_weather);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setTitle("Weather");

        queue = Volley.newRequestQueue(this);

        cityText = (TextView) findViewById(R.id.cityText);
        coordsText = (TextView) findViewById(R.id.coordsText);
        countryText = (TextView) findViewById(R.id.countryText);
        tempText = (TextView) findViewById(R.id.tempText);
        descText = (TextView) findViewById(R.id.descText);
        weatherIcon = (ImageView) findViewById(R.id.weatherIcon);
        forecastLayout = (LinearLayout) findViewById(R.id.forecastLayout);
        forecastTitle = (TextView) findViewById(R.id.forecastTitle);

        Bundle extras = getIntent().getExtras();
        if (extras != null){
            if(extras.getString("TYPE").equals("COORDS")){
                double lat = extras.getDouble("LAT");
                double lon = extras.getDouble("LON");
                getWeatherDataByCoords(lat, lon);
            }
            else if(extras.getString("TYPE").equals("CITY")){
                getWeatherByCity(extras.getString("NAME").toString());
            }
            else if(extras.getString("TYPE").equals("RANDOM")){
                int index = random.nextInt(capitals.size());
                getWeatherByCity(capitals.get(index));
            }
        }
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    private void getWeatherDataByCoords(double lat, double lon){
        String urlParams = "lat=" + String.valueOf(lat) + "&lon=" + String.valueOf(lon);
        sendApiRequest(urlParams, "weather?", null);
        sendApiRequest(urlParams, "forecast?", null);
    }

    private void getWeatherByCity(String cityName){
        cityName.replace(" ", "+");
        String urlParams = "type=like&q=" + cityName;
        sendApiRequest(urlParams, "weather?", cityName);
        sendApiRequest(urlParams, "forecast?", cityName);
    }

    /*private void getWeatherByCityId(int cityId){
        String urlParams = "id=" + cityId;
        sendApiRequest(urlParams, "weather?");
        sendApiRequest(urlParams, "forecast?");
    }*/

    public String sendApiRequest(String urlParams, String type, final String city){
        String fullUrl = baseUrl + type + "&" + units + "&" + key + "&" + urlParams;
        final String reqType = type;

        JsonObjectRequest stringRequest = new JsonObjectRequest(Request.Method.GET, fullUrl, null,
                new Response.Listener<JSONObject>() {
                    @Override
                    public void onResponse(JSONObject response) {
                        try {
                            if(reqType.equals("weather?")){
                                updateUI(response);
                            }
                            else if(reqType.equals("forecast?")){
                                updateForecast(response);
                            }

                        }
                        catch(Exception ex){
                            ex.printStackTrace();
                        }
                    }
                }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                if(error.networkResponse.statusCode == 404){
                    new AlertDialog.Builder(context)
                                .setMessage("City not found: " + city)
                                .setNeutralButton("OK", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialogInterface, int i) {
                                        WeatherActivity.super.onBackPressed();
                                    }
                                })
                        .show();
                }
                Log.d("VOLLEY", "Hups: " + error.getMessage());
            }
        });

        queue.add(stringRequest);
        return null;
    }


    private void updateUI(JSONObject response){
        try {
            String city = response.getString("name");
            String coordinates = "(" + response.getJSONObject("coord").getString("lat") + "  " +
                    response.getJSONObject("coord").getString("lon") + ")";
            String countryCode = response.getJSONObject("sys").getString("country");
            String description = response.getJSONArray("weather").getJSONObject(0).getString("main");
            double accurateTemp = response.getJSONObject("main").getDouble("temp");
            String displayTemp = String.format("%.1f", accurateTemp) + " ºC";
            int descId = response.getJSONArray("weather").getJSONObject(0).getInt("id");

            cityText.setText(city);
            coordsText.setText(coordinates);
            countryText.setText(getCountryFromCode(countryCode));
            tempText.setText(displayTemp);

            descText.setText(description);
            weatherIcon.setImageResource(getWeatherIcon(descId, 16));
        }
        catch(Exception ex){
            ex.printStackTrace();
        }
    }

    private void updateForecast(JSONObject response){
        try{
            JSONArray weatherArray = response.getJSONArray("list");
            for(int i = 1; i < weatherArray.length(); i++){
                JSONObject weatherItem = weatherArray.getJSONObject(i);

                String forcAccurateDate = weatherItem.getString("dt_txt");
                Date wtfJava = new SimpleDateFormat("yyyy-MM-dd HH:mm").parse(forcAccurateDate);
                String forcDisplayDate = new SimpleDateFormat("d.M").format(wtfJava);
                String forcDisplayClock = new SimpleDateFormat("HH:mm").format(wtfJava);

                int forcIcon = getWeatherIcon(weatherItem.getJSONArray("weather").getJSONObject(0).getInt("id"), wtfJava.getHours());
                double forcAccurateTemp = weatherItem.getJSONObject("main").getDouble("temp");
                String forcDisplayTemp = String.format("%.0f", forcAccurateTemp) + " ºC";

                forecastLayout.addView(createForecastColumn(forcDisplayDate, forcDisplayClock ,forcIcon, forcDisplayTemp));
            }
            forecastTitle.setVisibility(View.VISIBLE);
        }

        catch(Exception e){
            e.printStackTrace();
        }
    }

    private View createForecastColumn(String date, String clock, int image, String temp){
        LayoutInflater inflater = LayoutInflater.from(this);
        View column = inflater.inflate(R.layout.forecast_column, null, false);

        TextView forecastDate = (TextView) column.findViewById(R.id.forecastDate);
        TextView forecastClock = (TextView) column.findViewById(R.id.forecastClock);
        ImageView forecastIcon = (ImageView) column.findViewById(R.id.forecastIcon);
        TextView forecastTemp = (TextView) column.findViewById(R.id.forecastTemp);

        forecastDate.setText(date);
        forecastClock.setText(clock);
        forecastIcon.setImageResource(image);
        forecastTemp.setText(temp);

        return column;
    }

    // Statuses from https://openweathermap.org/weather-conditions
    // Icons from https://icons8.com/
    private int getWeatherIcon(int conditionId, int time){
        if (conditionId >= 200 && conditionId < 300) {
            return R.drawable.thunderstorm;
        }
        else if (conditionId >= 300 && conditionId <= 399){
            return R.drawable.drizzle;
        }
        else if (conditionId >= 500 && conditionId <= 599){
            return R.drawable.heavyrain;
        }
        else if (conditionId >= 600 && conditionId <= 699){
            return R.drawable.snow;
        }
        else if (conditionId >= 700 && conditionId <= 799){
            return R.drawable.mist;
        }
        else if (conditionId == 800) {
            if (time >= 9 && time <= 18) {
                return R.drawable.sunny;
            } else {
                return R.drawable.night;
            }
        }
        else if (conditionId >= 801 && conditionId <= 899){
            return R.drawable.cloudy;
        }
        else if (conditionId >= 900 && conditionId <= 906){
            return R.drawable.extreme;
        }
        else if (conditionId >= 951 && conditionId <= 962){
            return R.drawable.windy;
        }
        else {
            return R.drawable.unknown;
        }
    }

    // https://stackoverflow.com/questions/14275868/get-country-name-from-country-code
    private String getCountryFromCode(String countryCode){
        Locale loc = new Locale("",countryCode);
        return loc.getDisplayCountry();
    }

}
